<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

class Perfectmoney extends PaymentModule {

    protected $_html = '';
    protected $_postErrors = array();

    public function __construct() {

        $this->name = 'perfectmoney';
        $this->tab = 'payments_gateways';
        $this->version = '1.2.0';
        $this->ps_versions_compliancy = array('min' => '1.7.0.0', 'max' => _PS_VERSION_);
        $this->author = 'SR&I';
        $this->controllers = array('payment');

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = 'Perfect Money';
        $this->description = $this->trans('Perfect Money payment module', array(), 'Modules.Perfectmoney.Admin');
        $this->confirmUninstall = $this->trans('Are you sure you want to uninstall Perfect Money payment module?', array(), 'Modules.Perfectmoney.Admin');
    }

    public function install() {

        if (!parent::install() || !$this->registerHook('paymentOptions'))
            return false;

        return true;
    }

    public function uninstall() {

        if (!parent::uninstall() ||
                !Configuration::deleteByName('PERFECTMONEY_ACCOUNT') ||
                !Configuration::deleteByName('PERFECTMONEY_NAME') ||
                !Configuration::deleteByName('PERFECTMONEY_PASSPHRASE') ||
                !$this->unregisterHook('paymentOptions')) {
            return false;
        }
        return true;
    }

    public function getConfigFieldsValues() {
        return array(
            'PERFECTMONEY_ACCOUNT' => Tools::getValue('PERFECTMONEY_ACCOUNT', Configuration::get('PERFECTMONEY_ACCOUNT')),
            'PERFECTMONEY_NAME' => Tools::getValue('PERFECTMONEY_NAME', Configuration::get('PERFECTMONEY_NAME')),
            'PERFECTMONEY_PASSPHRASE' => Tools::getValue('PERFECTMONEY_PASSPHRASE', Configuration::get('PERFECTMONEY_PASSPHRASE'))
        );
    }

    public function renderForm() {

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Settings', array(), 'Modules.Perfectmoney.Admin'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->trans('PM account', array(), 'Modules.Perfectmoney.Admin'),
                        'name' => 'PERFECTMONEY_ACCOUNT',
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Payee name', array(), 'Modules.Perfectmoney.Admin'),
                        'name' => 'PERFECTMONEY_NAME',
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Alternate PassPhrase', array(), 'Modules.Perfectmoney.Admin'),
                        'name' => 'PERFECTMONEY_PASSPHRASE',
                        'required' => true
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions'),
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? : 0;
        $this->fields_form = array();
        $helper->id = (int) Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure='
                . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getContent() {

        $this->_html = '<h2>Perfect Money</h2>';

        if (Tools::isSubmit('btnSubmit')) {

            if (empty($_POST['PERFECTMONEY_ACCOUNT']))
                $this->_postErrors[] = $this->trans('PM account ID is required', array(), 'Modules.Perfectmoney.Admin');
            if (empty($_POST['PERFECTMONEY_NAME']))
                $this->_postErrors[] = $this->trans('Payee name is required', array(), 'Modules.Perfectmoney.Admin');
            if (empty($_POST['PERFECTMONEY_PASSPHRASE']))
                $this->_postErrors[] = $this->trans('Alternate PassPhrase is required', array(), 'Modules.Perfectmoney.Admin');

            if (!sizeof($this->_postErrors)) {

                Configuration::updateValue('PERFECTMONEY_ACCOUNT', strval($_POST['PERFECTMONEY_ACCOUNT']));
                Configuration::updateValue('PERFECTMONEY_NAME', strval($_POST['PERFECTMONEY_NAME']));
                Configuration::updateValue('PERFECTMONEY_PASSPHRASE', strval($_POST['PERFECTMONEY_PASSPHRASE']));
                $this->_html .= $this->displayConfirmation($this->trans('Settings updated', array(), 'Admin.Global'));
            } else {

                foreach ($this->_postErrors as $err) {
                    $this->_html .= $this->displayError($err);
                }
            }
        }

        $this->_html .= $this->displayPerfectMoney();
        $this->_html .= $this->renderForm();

        return $this->_html;
    }

    public function displayPerfectMoney() {

        $this->_html .= '<div class="alert alert-info">
<img src="../modules/perfectmoney/logo.png" style="float:left; margin-right:15px;" height="60">
<p><b>This module allows you to accept payments by Perfect Money.</b></p>
<p>Your PM account is automatically credited upon customer payment.</p>
<p>To start using this module please configure your Perfect Money account.</p>
</div>';
    }

    public function hookPaymentOptions($params) {

        // return payment options

        $payment_option = new PaymentOption();

        $payment_controller = $this->context->link->getModuleLink(
                $this->name, 'payment', array(), true
        );

        $payment_option->setCallToActionText($this->displayName)
                ->setAction($payment_controller);

        $payment_options[] = $payment_option;

        return $payment_options;
    }

    public function validateOrder(
    $id_cart, $id_order_state, $amount_paid, $payment_method = 'Unknown', $message = null, $extra_vars = [], $currency_special = null, $dont_touch_amount = false, $secure_key = false, Shop $shop = null
    ) {

        if (!$this->active)
            return;

        parent::validateOrder($id_cart, $id_order_state, $amount_paid);
    }

    public function changeOrderStatus($order_obj, $status, $order_currency, $error_msg = "") {

        Context::getContext()->currency = $order_currency;

        $order_obj->setCurrentState($status);

        if ($status == _PS_OS_ERROR_)
            PrestaShopLogger::addLog($error_msg, 1, null, 'Order');
    }

}